@@ -5,13 +5,12 @@
 #' is based on java. The .jar File started with this function is
 #' in the directory where SPOT is installed. 
 #'
-#' Java runtime environment needs to be installed to use the gui.
+#' Java runtime environment needs to be installed to use the GUI.
 #'
-#' @seealso  \code{\link{SPOT}}
 #' @export
 ####################################################################################
 spotGui<-function(){
-	guiPath=.find.package("SPOT");
+	guiPath=find.package("SPOT");
 	guiPath=file.path(guiPath, "GUI/spotGui.jar")
 	system(paste("java -jar ",guiPath))
 }
